/**
 * Object einer Card
 */
public class Card {
    private String name;
    private double damage;
    private ElementTyp elementTyp;
    private CardType cardType;

    /**
     * Erstellt eine neue Card mit gegebenen Eigenschaften
     * @param name Name der Card
     * @param damage Damage den die Card macht
     * @param elementTyp ElementType der Card
     * @param cardType CardType der Card
     */
    public Card(String name, double damage, ElementTyp elementTyp, CardType cardType) {
        this.name = name;
        this.damage = damage;
        this.elementTyp = elementTyp;
        this.cardType = cardType;
    }

    /**
     * Holt den Namen der Card
     * @return Name der Card
     */
    public String getName() {
        return this.name;
    }

    /**
     * Holt den Damage der Card
     * @return Damage der Card
     */
    public double getDamage() {
        return this.damage;
    }

    /**
     * Holt den ElementType der Card
     * @return ElementType der Card
     */
    public ElementTyp getElementTyp() {
        return this.elementTyp;
    }

    /**
     * Holt den CardType der Card
     * @return CardType der Card
     */
    public CardType getCardType() {
        return this.cardType;
    }

    /**
     * Berechnet die Wirksamkeit gegen eine Andere card. Die Card im Parameter wird angegriffen
     * @param toCeck Card gegen welche die Wirksamkeit geprüft werden soll
     * @return Gibt die Wirksamkeit zurück
     * @throws InternalError Wenn die Effectifeness von den 2 Cards nicht berechnet werden kann
     */
    public EffectivnessType getEffectivenessAgainst(Card toCeck) {
        if(toCeck.getCardType() == CardType.MONSTER){
            if(this.cardType == CardType.MONSTER){
                return EffectivnessType.NOEFFECT;
            }else if (this.cardType == CardType.SPELL){

            }
        }else if (toCeck.getCardType() == CardType.SPELL){
            if ((toCeck.getElementTyp() == ElementTyp.FIRE && this.elementTyp == ElementTyp.WATER) || (toCeck.getElementTyp() == ElementTyp.NORMAL && this.elementTyp == ElementTyp.FIRE) || (toCeck.getElementTyp() == ElementTyp.WATER && this.elementTyp == ElementTyp.NORMAL)){
                return EffectivnessType.EFFECTIVE;
            }else if (this.elementTyp == ElementTyp.FIRE && toCeck.getElementTyp() == ElementTyp.WATER){
                return EffectivnessType.NOTEFFECTIVE;
            }else if ((toCeck.getElementTyp() == ElementTyp.NORMAL && this.cardType == CardType.MONSTER && this.elementTyp == ElementTyp.NORMAL)){
                return EffectivnessType.NOEFFECT;
            }
        }
        throw new InternalError("Konnte die Effectiveness von den Cards nicht berechnen");
    }

    /**
     * Setztden namen der Card
     * @param neuerName Name der Card
     */
    public void setName(String neuerName) {
        this.name = name;
    }

    /**
     * Setzt den Damage der Card
     * @param damage Der Damage
     */
    public void setDamage(int damage) {
        this.damage = damage;
    }

    /**
     * Setzt den ElementType der Card
     * @param elementTyp Der ElementType
     */
    public void setElementType(ElementTyp elementTyp) {
        this.elementTyp = elementTyp;
    }

    /**
     * Testet ob die beiden Karten gleich sind
     * @param card Card die geprüft werden soll
     * @return True wenn die Karten übereinstimmen
     */
    public boolean equals(Card card){
        if(card == null) return false;
        if(this.name == card.getName() && this.cardType == card.getCardType() && this.elementTyp == card.getElementTyp() && this.damage == card.getDamage()){
            return true;
        }else {
            return false;
        }
    }
}
