import java.util.ArrayList;
import java.util.concurrent.atomic.AtomicBoolean;

/**
 * Verwaltet eine Liste vo Cards
 */
public class Cards {
    private ArrayList<Card> cards;

    /**
     * Erstellt ein nues obj mit gegebenen Karten
     * @param cardsArrayList Die Cards zum erstellen des Obj
     */
    public Cards(ArrayList<Card> cardsArrayList) {
        this.cards = cardsArrayList;
    }

    /**
     * Erstellt ein neues Obj ohne Cards
     */
    public Cards() {

    }

    /**
     * Fügt eine neue Card hinzu
     * @param newCard neue Card
     */
    public void addCard(Card newCard) {
        this.cards.add(newCard);
    }

    /**
     * Holt alle Cards
     * @return Alle Cards
     */
    public ArrayList<Card> getCards() {
        return this.cards;
    }

    /**
     * Löscht die gegebene Card
     * @param delCard Card zum löschen
     */
    public void delCard(Card delCard) {
        this.cards.removeIf(obj -> obj.equals(delCard));
    }

    /**
     * Prüft ob eine Card in den Cards vorhanden ist
     * @param toCeck Card nach der in den Cards gesucht werden soll
     * @return True wenn die Card in den Cards ist
     */
    public boolean containsCard(Card toCeck){
        AtomicBoolean returnval = new AtomicBoolean(false);
        this.cards.forEach(item -> {
            if(item.getElementTyp().equals(toCeck.getElementTyp()) && item.getCardType().equals(toCeck.getCardType()) && item.getName().equals(toCeck.getName()) && item.getDamage() == toCeck.getDamage()){
                returnval.set(true);
            }
        });
        return returnval.get();
    }

    /**
     * Vergleicht 2 Cards Obj miteinander
     * @param toCompare Cards zum vergleichen
     * @return True wenn es aus den selben Cards besteht
     */
    public boolean equals(Cards toCompare){
        AtomicBoolean returnval = new AtomicBoolean(false);
        this.cards.forEach(item -> {
            if(toCompare.containsCard(item)){
                returnval.set(true);
            }
        });
        return returnval.get();
    }
}
