/**
 * Ein einzelner Trading deal wird hier verwaltet.
 */
public class TradingDeal {
    private User tradingUser;
    private Card cardToTrade;
    private ElementTyp requiredElementType;
    private double requiredMinDamage;
    private CardType requiredCardType;

    /**
     * Ein TradingDeal wird  erstellt
     * @param user User des Deals
     * @param cardToTrade Card die getauscht werden soll
     * @param requiredElementType Req ElementType für den Tausch
     * @param requiredMinDamage Req min damage für den Tausch
     * @param requiredCardType Req Card Type für den Tausch
     */
    public TradingDeal(User user, Card cardToTrade, ElementTyp requiredElementType, double requiredMinDamage, CardType requiredCardType) {
        this.tradingUser = user;
        this.cardToTrade = cardToTrade;
        this.requiredElementType = requiredElementType;
        this.requiredMinDamage = requiredMinDamage;
        this.requiredCardType = requiredCardType;
    }

    /**
     * Holt den Trading User
     * @return User der den Trade erstellt hat
     */
    public User getTradingUser() {
        return this.tradingUser;
    }

    /**
     * Card die getraded werden soll
     * @return Card die getradet werden soll
     */
    public Card getCardToTrade() {
        return this.cardToTrade;
    }

    /**
     * Req Element type holen
     * @return Req Element Type
     */
    public ElementTyp getRequiredElementType() {
        return this.requiredElementType;
    }

    /**
     * Req min damage holen
     * @return Req min damage
     */
    public double getRequiredMinDamage() {
        return this.requiredMinDamage;
    }

    /**
     * Hole req cardtype
     * @return Req CardType
     */
    public CardType getRequiredCardType() {
        return this.requiredCardType;
    }

    /**
     * Setzt den User des Trades
     * @param tradingUser User der Traded
     */
    public void setTradingUser(User tradingUser) {
        this.tradingUser = tradingUser;
    }

    /**
     * Setzt die Karte die Getradet werden soll
     * @param cardToTrade Card zum tauschen
     */
    public void setCardToTrade(Card cardToTrade) {
        this.cardToTrade = cardToTrade;
    }

    /**
     * Setzt req elementType
     * @param requiredElementType Der Req elementType
     */
    public void setRequiredElementType(ElementTyp requiredElementType) {
        this.requiredElementType = requiredElementType;
    }

    /**
     * Setzt die req Min damage
     * @param requiredMinDamage Die req min gamage
     */
    public void setRequiredMinDamage(double requiredMinDamage) {
        this.requiredMinDamage = requiredMinDamage;
    }

    /**
     * Setzt req cardType
     * @param requiredCardType Der Req CardType
     */
    public void setRequiredCardType(CardType requiredCardType) {
        this.requiredCardType = requiredCardType;
    }

    /**
     * Vom deal im Parameter wird die Card mit den Requirements von THIS verglichen
     * @param checkDeal Von diesem Deal wird die CardToTrade geprüft
     * @return True wenn der Deal passt
     */
    public boolean dealOk(TradingDeal checkDeal){
        return this.cardOk(checkDeal.getCardToTrade());
    }

    /**
     * Von der Card im Parameter wird die Card mit den Requirements von THIS verglichen
     * @param checkCard Card die mit dem Deal verglichen wird
     * @return True wenn der Deal passt
     */
    public boolean cardOk(Card checkCard){
        if(checkCard.getCardType().equals(this.requiredCardType) && checkCard.getElementTyp().equals(this.requiredElementType) && checkCard.getDamage() >= this.requiredMinDamage){
            return true;
        }else {
            return false;
        }
    }
}
