import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import java.util.ArrayList;

import static org.junit.jupiter.api.Assertions.assertTrue;

public class StoreTest {

    private TradingDeal tradingDeal;
    private Cards newCards, cards, nochNewCards;
    private Card newCard, card;
    private ArrayList<Card> cardsArrayList, cardsArrayList2, cardsArrayList3;
    private Stack stack;
    private User user, userNew;
    private Store store;

    @BeforeEach
    void setUp() {
        //Stack
        card = new Card("Name", 10, ElementTyp.WATER, CardType.MONSTER);
        newCard = new Card("NameNew", 10, ElementTyp.WATER, CardType.SPELL);
        cardsArrayList = new ArrayList<Card>();
        cardsArrayList2 = new ArrayList<Card>();
        cardsArrayList3 = new ArrayList<Card>();
        cardsArrayList.add(card);
        cardsArrayList2.add(newCard);
        cards = new Cards(cardsArrayList);
        newCards = new Cards(cardsArrayList2);
        nochNewCards = new Cards(cardsArrayList3);
        stack = new Stack(cards, nochNewCards);

        //Coins
        Coins coins = new Coins(10);
        //User
        Credentials credentials = new Credentials("username", "passwort");
        user = new User(credentials, "name", "nachname", "email", stack, coins);
        userNew = new User(credentials, "nameNew", "nachnameNew", "emailNew", stack, coins);

        //Trading Deal
        Card cardToTrade = card;
        ElementTyp requiredElementType = ElementTyp.FIRE;
        double requiredMinDamage = 100;
        CardType requiredCardType = CardType.MONSTER;
        tradingDeal = new TradingDeal(user, cardToTrade, requiredElementType, requiredMinDamage, requiredCardType);

    }

    @Test
    void test_addTradingDeal() {
        user.addTradingDeal(tradingDeal);
        ArrayList<TradingDeal> expected = new ArrayList<TradingDeal>();
        expected.add(tradingDeal);
        Card c1 = new Card("Name", 10, ElementTyp.FIRE, CardType.MONSTER);
        Card c2 = new Card("Name1", 12, ElementTyp.FIRE, CardType.SPELL);
        Card c3 = new Card("Name2", 13, ElementTyp.WATER, CardType.MONSTER);
        Card c4 = new Card("Name3", 14, ElementTyp.WATER, CardType.SPELL);
        Card c5 = new Card("Name4", 15, ElementTyp.FIRE, CardType.MONSTER);
        Cards deck = new Cards();
        Cards stack = new Cards();
        deck.addCard(c1);
        deck.addCard(c2);
        stack.addCard(c3);
        stack.addCard(c4);
        stack.addCard(c5);
        TradingDeal vonDB = new TradingDeal(new User(new Credentials("username", "passwort"), "Name", "Nachname", "Email", new Stack(stack, deck), new Coins(1000)), c1, ElementTyp.FIRE, 10, CardType.SPELL);
        expected.add(vonDB);
        assertTrue(user.getTradingDeals().equals(expected));
    }
    @Test
    void test_doTrade() {
        Card cardtest = new Card("Name", 10, ElementTyp.FIRE, CardType.MONSTER);
        TradingDeal tradingDealtest = new TradingDeal(user, cardtest, ElementTyp.WATER, 100, CardType.MONSTER);
        boolean result = user.doTrading(tradingDeal, tradingDealtest);
        assertTrue(result);
    }
    @Test
    void test_getTradingDeals() {
        ArrayList<TradingDeal> result = user.getTradingDeals();
        ArrayList<TradingDeal> expected = new ArrayList<TradingDeal>();
        Card c1 = new Card("Name", 10, ElementTyp.FIRE, CardType.MONSTER);
        Card c2 = new Card("Name1", 12, ElementTyp.FIRE, CardType.SPELL);
        Card c3 = new Card("Name2", 13, ElementTyp.WATER, CardType.MONSTER);
        Card c4 = new Card("Name3", 14, ElementTyp.WATER, CardType.SPELL);
        Card c5 = new Card("Name4", 15, ElementTyp.FIRE, CardType.MONSTER);
        Cards deck = new Cards();
        Cards stack = new Cards();
        deck.addCard(c1);
        deck.addCard(c2);
        stack.addCard(c3);
        stack.addCard(c4);
        stack.addCard(c5);
        TradingDeal vonDB = new TradingDeal(new User(new Credentials("username", "passwort"), "Name", "Nachname", "Email", new Stack(stack, deck), new Coins(1000)), c1, ElementTyp.FIRE, 10, CardType.SPELL);
        expected.add(vonDB);
        expected.add(tradingDeal);
        assertTrue(result.equals(expected));
    }

    @Test
    void test_getPossibleTradingDeals() {
        Card testCard = new Card("Test", 15, ElementTyp.FIRE, CardType.SPELL);
        ArrayList<TradingDeal> result = user.getPossibleTradingDeals(testCard);
        ArrayList<TradingDeal> expected = new ArrayList<TradingDeal>();
        Card c1 = new Card("Name", 10, ElementTyp.FIRE, CardType.MONSTER);
        Card c2 = new Card("Name1", 12, ElementTyp.FIRE, CardType.SPELL);
        Card c3 = new Card("Name2", 13, ElementTyp.WATER, CardType.MONSTER);
        Card c4 = new Card("Name3", 14, ElementTyp.WATER, CardType.SPELL);
        Card c5 = new Card("Name4", 15, ElementTyp.FIRE, CardType.MONSTER);
        Cards deck = new Cards();
        Cards stack = new Cards();
        deck.addCard(c1);
        deck.addCard(c2);
        stack.addCard(c3);
        stack.addCard(c4);
        stack.addCard(c5);
        TradingDeal vonDB = new TradingDeal(new User(new Credentials("username", "passwort"), "Name", "Nachname", "Email", new Stack(stack, deck), new Coins(1000)), c1, ElementTyp.FIRE, 10, CardType.SPELL);
        expected.add(vonDB);
        assertTrue(result.equals(expected));
    }
}
